import java.io.*;

// Important note: the correct exception handling is omitted
// through all of these examples of serialization.
class App {
	
	private static final String SETTINGS_PATH = "settings.dat";
	private static final String DOCUMENT_PATH = "document.dat";
	
	public static void main(String[] args)
		throws IOException, ClassNotFoundException {
		// Test serialization on different objects
		testSaveSettings();
		testRestoreSettings();
		testSaveAndRestoreSettingsMultiple();
		testSaveDocument();
		testRestoreDocument();
	}
	
	private static void testSaveSettings() throws IOException {
		// Just prepare a compound object
		Settings s = new Settings();
		s.historySize = 10;
		s.fontSettings = new FontSettings("Arial");
		s.fontSettings.fontSize = 42;
		// Note that we invoke the setter once explicitly
		s.fontSettings.setBold(true);
		
		// Serialize it into the stream
		ObjectOutputStream oos = getOutputStream(SETTINGS_PATH);
		oos.writeObject(s);
		oos.close();
	}
	
	private static void testRestoreSettings()
		throws IOException, ClassNotFoundException {
		ObjectInputStream ois = getInputStream(SETTINGS_PATH);
		// Explicit cast needed
		Settings s = (Settings)ois.readObject();
		ois.close();
		
		System.out.println("Restored settings: " + s);
	}
	
	private static void testSaveAndRestoreSettingsMultiple()
		throws IOException, ClassNotFoundException {
		Settings s = new Settings();
		s.historySize = 20;
		
		ObjectOutputStream oos = getOutputStream(SETTINGS_PATH);
		// Serialization will cache the object reference
		oos.writeObject(s);
		
		// Try it out: reset the reference cache
		// oos.reset();
		
		// The second write will not serialize the
		// value again if it's already in the cache
		s.historySize = 30;
		oos.writeObject(s);
		
		oos.close();
		
		ObjectInputStream ois = getInputStream(SETTINGS_PATH);
		System.out.println("Restored settings 1: " + ois.readObject());
		System.out.println("Restored settings 2: " + ois.readObject());
		ois.close();
	}
	
	private static void testSaveDocument() throws IOException {
		// Just prepare a compound object
		Document d = new Document();
		d.comment = "This is a test document.";
		Node root = d.tree = new Node(0, null);
		root.left = new Node(1, root);
		root.right = new Node(2, root);
		
		// Serialize it into the stream
		ObjectOutputStream oos = getOutputStream(DOCUMENT_PATH);
		oos.writeObject(d);
		oos.close();
	}
	
	private static void testRestoreDocument()
		throws IOException, ClassNotFoundException {
		ObjectInputStream ois = getInputStream(DOCUMENT_PATH);
		// Explicit cast needed
		Document d = (Document)ois.readObject();
		ois.close();
		
		System.out.println("Restored document: " + d);
	}
	
	private static ObjectOutputStream getOutputStream(String fileName)
		throws IOException {
		FileOutputStream fso = new FileOutputStream(fileName);
		return new ObjectOutputStream(fso);
	}
	
	private static ObjectInputStream getInputStream(String fileName)
		throws IOException {
		FileInputStream fsi = new FileInputStream(fileName);
		return new ObjectInputStream(fsi);
	}
}
