
import java.text.SimpleDateFormat;
import java.util.GregorianCalendar;
import java.util.List;
import javax.persistence.*;
import entities.*;

public class Main {

    private static EntityManager em;
    private static int hrId;
    private static int financesId;

    public static void main(String[] args) {
        EntityManagerFactory emf = Persistence.createEntityManagerFactory("testpu");
        em = emf.createEntityManager();
        em.getTransaction().begin();

        // Create department entities
        createDepartments();
        // Add employees and parking lots into departments
        addEmployees();
        // Associate them with a meeting entity
        organizeMeeting();
        // Print out some query results to observe the behavior
        queryResults();

        em.getTransaction().commit();
        em.close();
        emf.close();
    }

    private static void createDepartments() {
        Department hr = new Department();
        hr.setName("Human Resources");
        em.persist(hr);
        hrId = hr.getId();

        Department finances = new Department();
        finances.setName("Finances");
        em.persist(finances);
        financesId = finances.getId();
    }

    private static void addEmployees() {
        // Get departments
        Department hr = em.find(Department.class, hrId);

        // The getReference method does not fetches the entity from the database,
        // it just returns a proxy, which can be used to create associations,
        // or to lazily load the entity data
        Department finances = em.getReference(Department.class, financesId);

        // Add employees to hr
        for (int i = 0; i < 5; ++i) {
            Employee emp = new Employee();
            emp.setName("HR worker #" + i);
            emp.setDepartment(hr);
            hr.addEmployee(emp);

            // HR employees also have a parking lot
            ParkingLot pl = new ParkingLot();
            pl.setLocation("Building A, slot #" + i);
            pl.setEmployee(emp);
            emp.setParkingLot(pl);

            // We dont need to persist ParkingLot separately, see cascadeType
            em.persist(emp);
        }

        // Add employees to finances
        for (int i = 0; i < 10; ++i) {
            Employee emp = new Employee();
            emp.setName("Finances worker #" + i);
            emp.setDepartment(finances);
            finances.addEmployee(emp);
            em.persist(emp);
        }
    }

    private static void organizeMeeting() {
        List<Employee> hrWorkers = em.createQuery(
            "SELECT e FROM Employee e WHERE e.department.name = 'Human Resources'",
            Employee.class).getResultList();

        Meeting hrMeeting = new Meeting();
        hrMeeting.setTitle("HR meeting");
        hrMeeting.setStart(new GregorianCalendar(2013, 01, 20, 12, 45));
        hrMeeting.setFinish(new GregorianCalendar(2013, 01, 20, 14, 15));
        em.persist(hrMeeting);

        for(Employee e: hrWorkers) {
            hrMeeting.addEmployee(e);
            e.addMeeting(hrMeeting);
        }
    }

    private static void queryResults() {
        // Force to send pending queries to the database
        em.flush();
        // Clear the internal entity collection (forces to re-fetch data)
        em.clear();

        // Select the name of each department, and the count of employees in them
        List<Object[]> deptStat = (List<Object[]>)em.createQuery(
            "SELECT d.name, COUNT(e) FROM Department d JOIN d.employees e GROUP BY d.name",
            Object[].class).getResultList();

        for(Object[] data : deptStat) {
            System.out.println("Department name: " + data[0]
                + ", employee count: " + data[1]);
        }

        // Select the employees from the HR department
        // Note that these should be ordered descending by name (@OrderBy)
        List<Employee> hrEmps = em.find(Department.class, hrId).getEmployees();

        for(Employee e: hrEmps) {
            System.out.println(e.getName() + ", parking lot: "
                + e.getParkingLot().getLocation() + ", meetings: ");

            // List all meetings
            SimpleDateFormat sdf = new SimpleDateFormat();
            for(Meeting m : e.getMeetings()) {
                System.out.println("\t" + m.getTitle()
                    + " (" + sdf.format(m.getStart().getTime())
                    + " - " + sdf.format(m.getFinish().getTime()) + ")");
            }
        }
    }
}
